import org.apache.http.client.methods.HttpPost;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 图像搜索，淘宝联盟接口
 */
public class API2Demo {
    private static final String ENCODING = "UTF-8";

    private static String percentEncode(String value) throws UnsupportedEncodingException {
        return value != null ? URLEncoder.encode(value, ENCODING).replace("+", "%20").replace("*", "%2A").replace("%7E", "~") : null;
    }

    private static final String ISO8601_DATE_FORMAT = "yyyy-MM-dd'T'HH:mm:ss'Z'";

    private static String formatIso8601Date(Date date) {
        SimpleDateFormat df = new SimpleDateFormat(ISO8601_DATE_FORMAT);
        df.setTimeZone(new SimpleTimeZone(0, "GMT"));
        System.out.println("date：" + df.format(date));
        return df.format(date);
    }

    static final Base64.Encoder encoder = Base64.getEncoder();
    private static String accessKeyId = "<your-access-key-id>";
    private static String keySecrect = "<your-access-key-secret>";
    private static String domain = "imagesearch.cn-shanghai.aliyuncs.com";

    public static void main(String[] args) {
        // 调用淘宝联盟searchByUrl接口
        testSearchByUrl();
        // 调用淘宝联盟getProductInfoByIds接口
        testGetProductInfoByIds();

    }

    public static void testSearchByUrl() {
        try {
            final String HTTP_METHOD = "POST";//这里传METHOD

            // 这里的参数涉及 到下文的 preUrl 拼接，需要参数数量一致，如果添加参数，下文也需要拼接
            Map parameters = new HashMap(12);
            // 输入请求参数
            parameters.put("Action", "SearchByUrl");//接口名
            parameters.put("Version", "2021-05-01");
            // 可以选择 JSON 或者 XML。默认为 XML。
            parameters.put("Format", "JSON");
            /**--***********---------****************************************************************/
            parameters.put("AccessKeyId", accessKeyId);
            /**--***********---------****************************************************************/
            parameters.put("Timestamp", formatIso8601Date(new Date()));
            parameters.put("SignatureMethod", "HMAC-SHA1");
            parameters.put("SignatureVersion", "1.0");
            parameters.put("SignatureNonce", UUID.randomUUID().toString());
            parameters.put("Accept", "application/json");
            parameters.put("RegionId", "cn-shanghai");
            /**--***********---------你的入参在下面****************************************************************/
            parameters.put("Pid", "pic值");
            parameters.put("PicUrl", "https://alibaba.oss-cn-shanghai.aliyuncs.com/123.jpg");
            // 其他参数参考 https://help.aliyun.com/document_detail/253965.html
            // parameters.put("Num", 10);


            String[] sortedKeys = (String[]) parameters.keySet().toArray(new String[]{});
            Arrays.sort(sortedKeys);
            final String SEPARATOR = "&";
            // 构造 stringToSign 字符串
            StringBuilder stringToSign = new StringBuilder();
            stringToSign.append(HTTP_METHOD).append(SEPARATOR);
            stringToSign.append(percentEncode("/")).append(SEPARATOR);
            StringBuilder canonicalizedQueryString = new StringBuilder();
            for (String key : sortedKeys) {
                // 这里注意编码 key 和 value
                canonicalizedQueryString.append("&")
                        .append(percentEncode(key)).append("=")
                        .append(percentEncode((String) parameters.get(key)));
            }
            // 这里注意编码 canonicalizedQueryString
            stringToSign.append(percentEncode(canonicalizedQueryString.toString().substring(1)));
//              Signature = Base64(HMAC-SHA1("KSOHryOxs3pB8u97zGDKpRQ8Gi7xmA", UTF-8-Encoding-Of(stringToSign)))
            final String ALGORITHM = "HmacSHA1";
            final String ENCODING = "UTF-8";
            /**--***********---------****************************************************************/
            String key = keySecrect + "&";
            /**--***********---------****************************************************************/
            String stringTo = stringToSign.toString();
            Mac mac = Mac.getInstance(ALGORITHM);
            mac.init(new SecretKeySpec(key.getBytes(ENCODING), ALGORITHM));
            byte[] signData = mac.doFinal(stringTo.getBytes(ENCODING));
            // 得到公共参数 Signature 值
            String signature = new String(encoder.encodeToString(signData));

            System.out.println("stringToSign:" + stringTo);
            System.out.println("signature:" + percentEncode(signature));
            System.out.println("SignatureNonce:" + parameters.get("SignatureNonce"));

            String preUrl = "http://imagesearch.cn-shanghai.aliyuncs.com/?"
                    + "SignatureVersion=" + parameters.get("SignatureVersion") + "&"
                    + "Action=" + parameters.get("Action")
                    + "&" + "Accept=" + parameters.get("Accept")
                    + "&" + "Format=" + parameters.get("Format")
                    + "&" + "SignatureNonce=" + parameters.get("SignatureNonce")
                    + "&" + "Version=" + parameters.get("Version")
                    + "&" + "AccessKeyId=" + parameters.get("AccessKeyId")
                    + "&" + "Signature=" + percentEncode(signature)
                    + "&" + "SignatureMethod=" + parameters.get("SignatureMethod")
                    + "&" + "Timestamp=" + parameters.get("Timestamp")
                    + "&" + "RegionId=" + parameters.get("RegionId");
            preUrl = preUrl + "&Pid=" + parameters.get("Pid") + "&PicUrl=" + parameters.get("PicUrl");

            HttpPost httpPost = new HttpPost(preUrl);
            System.out.println(preUrl);

            String result = ApiDemo.access(httpPost);
            System.out.println(result);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void testGetProductInfoByIds() {
        try {
            final String HTTP_METHOD = "POST";//这里传METHOD

            // 这里的参数涉及 到下文的 preUrl 拼接，需要参数数量一致，如果添加参数，下文也需要拼接
            Map parameters = new HashMap(12);
            // 输入请求参数
            parameters.put("Action", "GetProductInfoByIds");//接口名
            parameters.put("Version", "2021-05-01");
            // 可以选择 JSON 或者 XML。默认为 XML。
            parameters.put("Format", "JSON");
            /**--***********---------****************************************************************/
            parameters.put("AccessKeyId", accessKeyId);
            /**--***********---------****************************************************************/
            parameters.put("Timestamp", formatIso8601Date(new Date()));
            parameters.put("SignatureMethod", "HMAC-SHA1");
            parameters.put("SignatureVersion", "1.0");
            parameters.put("SignatureNonce", UUID.randomUUID().toString());
            parameters.put("Accept", "application/json");
            parameters.put("RegionId", "cn-shanghai");
            /**--***********---------你的入参在下面****************************************************************/
            parameters.put("Pid", "pic值");
            parameters.put("ItemIds", "548679889198");
            // 其他参数参考 https://help.aliyun.com/document_detail/446044.html


            String[] sortedKeys = (String[]) parameters.keySet().toArray(new String[]{});
            Arrays.sort(sortedKeys);
            final String SEPARATOR = "&";
            // 构造 stringToSign 字符串
            StringBuilder stringToSign = new StringBuilder();
            stringToSign.append(HTTP_METHOD).append(SEPARATOR);
            stringToSign.append(percentEncode("/")).append(SEPARATOR);
            StringBuilder canonicalizedQueryString = new StringBuilder();
            for (String key : sortedKeys) {
                // 这里注意编码 key 和 value
                canonicalizedQueryString.append("&")
                        .append(percentEncode(key)).append("=")
                        .append(percentEncode((String) parameters.get(key)));
            }
            // 这里注意编码 canonicalizedQueryString
            stringToSign.append(percentEncode(canonicalizedQueryString.toString().substring(1)));
//              Signature = Base64(HMAC-SHA1("KSOHryOxs3pB8u97zGDKpRQ8Gi7xmA", UTF-8-Encoding-Of(stringToSign)))
            final String ALGORITHM = "HmacSHA1";
            final String ENCODING = "UTF-8";
            /**--***********---------****************************************************************/
            String key = keySecrect + "&";
            /**--***********---------****************************************************************/
            String stringTo = stringToSign.toString();
            Mac mac = Mac.getInstance(ALGORITHM);
            mac.init(new SecretKeySpec(key.getBytes(ENCODING), ALGORITHM));
            byte[] signData = mac.doFinal(stringTo.getBytes(ENCODING));
            // 得到公共参数 Signature 值
            String signature = new String(encoder.encodeToString(signData));

            System.out.println("stringToSign:" + stringTo);
            System.out.println("signature:" + percentEncode(signature));
            System.out.println("SignatureNonce:" + parameters.get("SignatureNonce"));

            String preUrl = "http://imagesearch.cn-shanghai.aliyuncs.com/?"
                    + "SignatureVersion=" + parameters.get("SignatureVersion") + "&"
                    + "Action=" + parameters.get("Action")
                    + "&" + "Accept=" + parameters.get("Accept")
                    + "&" + "Format=" + parameters.get("Format")
                    + "&" + "SignatureNonce=" + parameters.get("SignatureNonce")
                    + "&" + "Version=" + parameters.get("Version")
                    + "&" + "AccessKeyId=" + parameters.get("AccessKeyId")
                    + "&" + "Signature=" + percentEncode(signature)
                    + "&" + "SignatureMethod=" + parameters.get("SignatureMethod")
                    + "&" + "Timestamp=" + parameters.get("Timestamp")
                    + "&" + "RegionId=" + parameters.get("RegionId");
            preUrl = preUrl + "&Pid=" + parameters.get("Pid") + "&ItemIds=" + parameters.get("ItemIds");

            HttpPost httpPost = new HttpPost(preUrl);
            System.out.println(preUrl);

            String result = ApiDemo.access(httpPost);
            System.out.println(result);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
