/**
 * Created by ithink on 18/5/8.
 */
import org.apache.commons.codec.binary.Base64;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpRequestBase;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;
import sun.misc.BASE64Encoder;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import java.io.*;
import java.nio.charset.Charset;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.text.SimpleDateFormat;
import java.util.*;

public class ApiDemo {
    private final static String CHARSET_UTF8 = "utf8";
    private final static String ALGORITHM = "UTF-8";

    private static String accessKeyId = "<your-access-key-id>";
    private static String keySecrect = "<your-access-key-secret>";

    private static String instanceName = "<your-instance-name>";

    private static String fileName = "./resources/test.jpg";

    private static String domain = "imagesearch.cn-shanghai.aliyuncs.com";

    public static void main(String[] args) throws Exception {
        if (accessKeyId.length() == 0 || keySecrect.length() == 0 || instanceName.length() == 0) {
            System.err.println("accessKeyId accessKeySecret and instanceName need specify.");
            return;
        }
        // 调用V2版本add接口
        testAddImage();
        // 调用V2版本search接口，里面有两种方式 SearchByPic 和 SearchByName
        testSearchImage();
        // 调用V2版本delete接口
        testDeleteImage();
    }

    public static void testAddImage() throws Exception {
        byte[] bytes2 = getBytes(fileName);
        Base64 base64 = new Base64();

        List<NameValuePair> params = new ArrayList<NameValuePair>(4);
        params.add(new BasicNameValuePair("InstanceName", instanceName));
        params.add(new BasicNameValuePair("PicName", "demo"));
        params.add(new BasicNameValuePair("ProductId", "demo"));
        params.add(new BasicNameValuePair("PicContent", base64.encodeToString(bytes2)));

        HttpEntity entity = new UrlEncodedFormEntity(params, CHARSET_UTF8);
        String content = convert(entity.getContent(), Charset.forName(CHARSET_UTF8));
        System.out.println(content);

        Map<String, String> headers = new HashMap<String, String>();
        String signatureStr = buildSignatureStr("add", content, headers);
        System.out.println(signatureStr);

        byte[] signBytes = hmacSHA1Signature(keySecrect, signatureStr);
        String signature = newStringByBase64(signBytes);
        String authorization = "acs " + accessKeyId + ":" + signature;
        headers.put("authorization", authorization);
        System.out.println(authorization);

        String url = "http://" + domain + "/v2/image/add";
        HttpPost httpPost = new HttpPost(url);
        for (String key : headers.keySet()) {
            httpPost.addHeader(key, headers.get(key));
        }

        httpPost.setEntity(entity);
        String result = access(httpPost);
        System.out.println(result);
    }

    public static void testSearchImage() throws Exception {
        byte[] bytes2 = getBytes(fileName);
        Base64 base64 = new Base64();

        List<NameValuePair> params = new ArrayList<NameValuePair>(4);
        params.add(new BasicNameValuePair("InstanceName", instanceName));
        params.add(new BasicNameValuePair("PicContent", base64.encodeToString(bytes2)));
        params.add(new BasicNameValuePair("Num", "1"));
        // Type = SearchByPic  说明请求的是SearchImageByPic，使用图片进行搜索。
        // Type = SearchByPic 时 PicContent 是必填
        params.add(new BasicNameValuePair("Type", "SearchByPic"));
        // Type = SearchByName  说明请求的是SearchImageByName，SearchByName
        // Type = SearchByName 时 ProductId和PicName，PicContent非必填
        // params.add(new BasicNameValuePair("Type", "SearchByName"));
        // params.add(new BasicNameValuePair("ProductId", "demo"));
        // params.add(new BasicNameValuePair("PicName", "demo"));

        HttpEntity entity = new UrlEncodedFormEntity(params, CHARSET_UTF8);
        String content = convert(entity.getContent(), Charset.forName(CHARSET_UTF8));
        System.out.println(content);

        Map<String, String> headers = new HashMap<String, String>();
        String signatureStr = buildSignatureStr("search", content, headers);
        System.out.println(signatureStr);

        byte[] signBytes = hmacSHA1Signature(keySecrect, signatureStr);
        String signature = newStringByBase64(signBytes);
        String authorization = "acs " + accessKeyId + ":" + signature;
        headers.put("authorization", authorization);
        System.out.println(authorization);

        StringBuilder curl = new StringBuilder();
        String url = "http://" + domain + "/v2/image/search";
        curl.append("curl -X POST \"").append(url).append("\"");
//        curl.append(" -d \"").append(content).append("\"");
        HttpPost httpPost = new HttpPost(url);
        for (String key : headers.keySet()) {
            httpPost.addHeader(key, headers.get(key));
        }

        httpPost.setEntity(entity);
        System.out.println(curl.toString());
        String result = access(httpPost);
        System.out.println(result);
    }

    public static void testDeleteImage() throws Exception {
        List<NameValuePair> params = new ArrayList<NameValuePair>(3);
        params.add(new BasicNameValuePair("InstanceName", instanceName));
        params.add(new BasicNameValuePair("PicName", "demo"));
        params.add(new BasicNameValuePair("ProductId", "demo"));

        HttpEntity entity = new UrlEncodedFormEntity(params, CHARSET_UTF8);
        String content = convert(entity.getContent(), Charset.forName(CHARSET_UTF8));

        Map<String, String> headers = new HashMap<String, String>();
        String signatureStr = buildSignatureStr("delete", content, headers);
        System.out.println(signatureStr);

        byte[] signBytes = hmacSHA1Signature(keySecrect, signatureStr);
        String signature = newStringByBase64(signBytes);
        String authorization = "acs " + accessKeyId + ":" + signature;
        headers.put("authorization", authorization);
        System.out.println(authorization);

        String url = "http://" + domain + "/v2/image/delete";
        HttpPost httpPost = new HttpPost(url);
        for (String key : headers.keySet()) {
            httpPost.addHeader(key, headers.get(key));
        }

        httpPost.setEntity(entity);
        String result = access(httpPost);
        System.out.println(result);
    }




    private static String getMd5(String body) throws NoSuchAlgorithmException, UnsupportedEncodingException {
        MessageDigest md5 = MessageDigest.getInstance("MD5");
        BASE64Encoder base64en = new BASE64Encoder();
        return base64en.encode(md5.digest(body.getBytes(CHARSET_UTF8)));
    }

    private static String getGMT() {
        Calendar cd = Calendar.getInstance();
        SimpleDateFormat sdf = new SimpleDateFormat("EEE d MMM yyyy HH:mm:ss 'GMT'", Locale.US);
        sdf.setTimeZone(TimeZone.getTimeZone("GMT"));
        return sdf.format(cd.getTime());
    }


    private static String generateSignatureNonce() {
        String base = "abcdefghijklmnopqrstuvwxyz0123456789";
        Random random = new Random();
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < 32; i++) {
            int number = random.nextInt(base.length());
            sb.append(base.charAt(number));
        }
        return sb.toString();
    }

    private static String buildSignatureStr(String operation, String postContent, Map<String, String> headers) {
        String data = "POST\n";
        String accept = "application/json";
        data += accept + "\n";

        String contentMd5;
        try {
            contentMd5 = getMd5(postContent);
        } catch (Exception e) {
            return null;
        }
        data += contentMd5 + "\n";

        String contentType = "application/x-www-form-urlencoded; charset=" + ALGORITHM;
        data += contentType + "\n";

        String gmt = getGMT();
        data += gmt + "\n";

        String method = "HMAC-SHA1";
        data += "x-acs-signature-method:" + method + "\n";

        String signatureNonce = generateSignatureNonce();
        data += "x-acs-signature-nonce:" + signatureNonce + "\n";

        String apiVersion = "2019-03-25";
        data += "x-acs-version:" + apiVersion + "\n";
        data += "/v2/image/" + operation;

        headers.put("x-acs-version", apiVersion);
        headers.put("x-acs-signature-method", method);
        headers.put("x-acs-signature-nonce", signatureNonce);

        headers.put("accept", accept);
        headers.put("content-md5", contentMd5);
        headers.put("content-type", contentType);
        headers.put("date", gmt);

        return data;
    }

    private static byte[] hmacSHA1Signature(String secret, String baseString) throws Exception {
        if (secret == null || secret.length() == 0) {
            throw new IOException("secret can not be empty");
        }
        if (baseString == null || baseString.length() == 0) {
            return null;
        }
        Mac mac = Mac.getInstance("HmacSHA1");
        SecretKeySpec keySpec = new SecretKeySpec(secret.getBytes(CHARSET_UTF8), ALGORITHM);
        mac.init(keySpec);
        return mac.doFinal(baseString.getBytes(CHARSET_UTF8));
    }

    private static String newStringByBase64(byte[] bytes)
        throws UnsupportedEncodingException {
        if (bytes == null || bytes.length == 0) {
            return null;
        }
        return new String(org.apache.commons.codec.binary.Base64.encodeBase64(bytes, false), CHARSET_UTF8);
    }

    public static String access(HttpRequestBase httpRequest) {
        CloseableHttpClient client = HttpClients.createDefault();
        HttpResponse response;
        String result;
        try {
            httpRequest.setHeader("accept-encoding", "");
            response = client.execute(httpRequest);
            if (response.getStatusLine().getStatusCode() == 200) {
                HttpEntity httpEntity = response.getEntity();
                result = EntityUtils.toString(httpEntity);
            } else {
                System.out.println(response.getStatusLine().getStatusCode());
                HttpEntity httpEntity = response.getEntity();
                result = EntityUtils.toString(httpEntity);
                System.out.println(result);
            }
        } catch (IOException e) {
            e.printStackTrace();
            return null;
        }

        return result;
    }

    private static String convert(InputStream inputStream, Charset charset) throws IOException {
        StringBuilder stringBuilder = new StringBuilder();
        String line;
        BufferedReader bufferedReader = null;
        try {
            bufferedReader = new BufferedReader(new InputStreamReader(inputStream, charset));
            while ((line = bufferedReader.readLine()) != null) {
                stringBuilder.append(line);
            }
        }finally {
            if (bufferedReader != null){
                bufferedReader.close();
            }
        }
        return stringBuilder.toString();
    }

    private static byte[] getBytes(String filePath) {
        byte[] buffer = null;
        try {
            File file = new File(filePath);
            FileInputStream fis = new FileInputStream(file);
            // picture max size is 2MB
            ByteArrayOutputStream bos = new ByteArrayOutputStream(2000 * 1024);
            byte[] b = new byte[1000];
            int n;
            while ((n = fis.read(b)) != -1) {
                bos.write(b, 0, n);
            }
            fis.close();
            bos.close();
            buffer = bos.toByteArray();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return buffer;
    }
}
